import csv
from io import StringIO
from logging import getLogger
from collections import Counter, defaultdict

logger = getLogger(__name__)

def count(functions:dict, table:iter) -> list:
    histograms = defaultdict(lambda: defaultdict(Counter))
    for row in table:
        for i, value in enumerate(row):
            for name, function in functions.items():
                try:
                    histograms[i][name][function(value)] += 1
                except Exception as e:
                    logger.error(e)
                    histograms[i][name][None] += 1
    return [histograms[i] for i in range(max(histograms.keys()) + 1)]

def load(table):
    if isinstance(table, str):
        fp = open(table, 'r')
        result = csv.reader(fp, dialect = _guess_dialect(fp))
    elif isinstance(table, StringIO):
        fp = table
        result = csv.reader(fp, dialect = _guess_dialect(fp))
    else:
        result = table
    return result
        
def _guess_dialect(fp):
    'Guess the dialect of a CSV file.'
    pos = fp.tell()
    try:
        dialect = csv.Sniffer().sniff(fp.read(1024))
    except csv.Error:
        dialect = 'excel' # the default
    fp.seek(pos)
    return dialect



from aamoer.pieces import count, load

def aamoer(functions:dict, table) -> list:
    '''
    Specify some function and an iterable of iterables
    (like a list of lists), where the inner iterable is
    a row in a data table.
    
    Receive histograms of the results of those functions.
    You receive one histogram per function per column.
    That is, if each row is eight long and you pass two
    functions, you'll get 16 histograms.
    '''
    return count(functions, load(table))

def cli(functions:dict):
    import sys, argparse, csv
    p = argparse.ArgumentParser(description = 'Find risk-related features in CSV files.')
    p.add_argument('csv', metavar = '[csv file]', nargs = '+', help = 'csv files to look at')
    writer = csv.writer(sys.stdout)
    writer.writerow(['filename','column.index','feature','value','count'])
    for filename in p.parse_args().csv:
        for column_number, column in enumerate(aamoer(functions, filename)):
            for feature_name, counter in column.items():
                for value, value_count in counter.items():
                    writer.writerow([filename,column_number,feature_name,value,value_count])



from distutils.core import setup

setup(name='aamoer',
      author='Thomas Levine',
      author_email='_@thomaslevine.com',
      description='Run statistics on data tables.',
      url='https://github.com/tlevine/aamoer',
      packages=['aamoer'],
      tests_require = ['nose'],
      version='0.0.1',
      license='AGPL',
)
